#![feature(specialization)]

#[macro_use]
extern crate pyo3;

use pyo3::prelude::*;

#[pyclass(module = "unit_converter")]
struct Temperature {
    celsius: f32,
}

#[pymethods]
impl Temperature {
    #[new]
    fn new(obj: &PyRawObject, temperature: f32) {
        obj.init(Temperature {
            celsius: temperature,
        });
    }

    fn to_fahrenheit(&self) -> f32 {
        self.celsius * 1.8 + 32.0
    }
}

#[pyfunction]
fn batch_converter(temperatures: Vec<f32>) -> Vec<f32> {
    temperatures
        .iter()
        .map(|temperature| {
            let temp = Temperature {
                celsius: temperature.clone(),
            };
            temp.to_fahrenheit()
        })
        .collect()
}

#[pymodule]
fn unit_converter(_py: Python, module: &PyModule) -> PyResult<()> {
    module.add_class::<Temperature>()?;
    module.add_wrapped(wrap_pyfunction!(batch_converter))?;
    Ok(())
}

#[cfg(test)]
mod tests {

    use crate::Temperature;

    #[test]
    fn conversion_celsius_to_fahrenheit() {
        let temperature = Temperature { celsius: 25.0 };
        assert_eq!(temperature.to_fahrenheit(), 77.0);
    }
}
